import java.util.Random;

public class ProcessingElement 
{
    static Random random = new Random();

    double[] weights;
    private static double [] sigtab = new double[10000];  // values of f(x) for x values 
    
    static 
    {
        for(int i=0; i<10000; i++)
        {
            // sum will be from a range of 10000 values between 0 and 10
            double sum = i/(double)1000;
            
            sigtab[i] = Sigmoid(sum);
        }
    }   

    
    public ProcessingElement(int inputs)
    {
        weights = new double[inputs];
        
        for(int i = 0; i < inputs; i++)
        {
            weights[i] = RandClamped();
        }
    }
    
    public ProcessingElement(double[] weightsIn)
    {
        weights = new double[weightsIn.length];
        
         for(int i = 0; i < weightsIn.length; i++)
         {
             weights[i] = weightsIn[i];
         }
    }
    
    public ProcessingElement(ProcessingElement processingElement)
    {
        weights = new double[processingElement.weights.length];
        
         for(int i = 0; i < processingElement.weights.length; i++)
         {
             weights[i] = processingElement.weights[i];
         }
    }
    
    public int getNumberInputs()
    {
        return weights.length;
    }
    
    public double getWeight(int index)
    {
        return weights[index];
    }
    
    public double getOutput(double[] inputs)
    {
        double summation = 0;
        
        for(int i = 0; i < weights.length; i++)
        {
            summation += weights[i] * inputs[i];
        }
        
        return myQuickSigmoid(summation);
        
        // return sigmoid(summation);
    }
    
	public static double Sigmoid(double inNet) 
	{
		if(inNet > 10)
			return 1;
		if(inNet < -10)
			return 0;
		
		return 1.0 / (1.0 +  (Math.exp(-inNet))); // http://kunuk.wordpress.com/2010/10/11/neural-network-backpropagation-with-java/
	}
    
    public static double myQuickSigmoid(double inNet)
	{
		if(inNet > 10)
			return 1;
		if(inNet < -10)
			return 0;
		
		double absolute = Math.abs(inNet);
		
		double index = absolute * 1000;
		
		if(index == 10000)
			index = 9999;
		
		if (inNet > 0)
			return sigtab[(int)index];
		else
			return 1 - (sigtab[(int)index]);
	}
    
    public static double RandClamped()
    {
        /*
        double first = random.nextDouble();
        double second = random.nextDouble();
        
        return (first - second);
        */

		
		double first = random.nextDouble();
		
		if(random.nextDouble() < 0.5)
			return (0 - first);
		return first;
    }
    
    public void mutate(double mutationRate, double maxPerturbation)
    {
        for(int i = 0; i < weights.length; i++)
        {
            if(random.nextDouble() < mutationRate)
            {
                weights[i] += (maxPerturbation * RandClamped());
            }
        }
    }
    
    public String getAsString()
    {
        String description = "";
        
        for(int i = 0; i < weights.length; i++)
        {
            description += weights[i] + ",";
        }
        
        return description;
    }
}
